@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyappfilechooser.c 9271 2009-06-17 11:20:34Z yeti-dn $
+ *  @(#) $Id: gwyappfilechooser.c 10215 2009-10-05 15:00:59Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -51,6 +51,7 @@ enum {
 typedef struct {
     GSList *list;
     GwyFileOperationType fileop;
+    gboolean only_nondetectable;
 } TypeListData;
 
 typedef struct {
@@ -72,7 +73,8 @@ static void       gwy_app_file_chooser_add_type       (const gchar *name,
 static gint       gwy_app_file_chooser_type_compare   (gconstpointer a,
                                                        gconstpointer b);
 static void       gwy_app_file_chooser_add_types      (GtkListStore *store,
-                                                       GwyFileOperationType fileop);
+                                                       GwyFileOperationType fileop,
+                                                       gboolean only_nondetectable);
 static void       gwy_app_file_chooser_add_type_list  (GwyAppFileChooser *chooser);
 static void       gwy_app_file_chooser_update_expander(GwyAppFileChooser *chooser);
 static void       gwy_app_file_chooser_type_changed   (GwyAppFileChooser *chooser,
@@ -159,6 +161,9 @@ gwy_app_file_chooser_add_type(const gchar *name,
     if (!(gwy_file_func_get_operations(name) & data->fileop))
         return;
 
+    if (data->only_nondetectable && gwy_file_func_get_is_detectable(name))
+        return;
+
     data->list = g_slist_prepend(data->list, (gpointer)name);
 }
 
@@ -172,7 +177,8 @@ gwy_app_file_chooser_type_compare(gconstpointer a,
 
 static void
 gwy_app_file_chooser_add_types(GtkListStore *store,
-                               GwyFileOperationType fileop)
+                               GwyFileOperationType fileop,
+                               gboolean only_nondetectable)
 {
     TypeListData tldata;
     GtkTreeIter iter;
@@ -180,6 +186,7 @@ gwy_app_file_chooser_add_types(GtkListStore *store,
 
     tldata.list = NULL;
     tldata.fileop = fileop;
+    tldata.only_nondetectable = only_nondetectable;
     gwy_file_func_foreach((GFunc)gwy_app_file_chooser_add_type, &tldata);
     tldata.list = g_slist_sort(tldata.list, gwy_app_file_chooser_type_compare);
 
@@ -411,19 +418,6 @@ gwy_app_file_chooser_type_changed(GwyAppFileChooser *chooser,
     g_free(chooser->filetype);
     gtk_tree_model_get(model, &iter, COLUMN_FILETYPE, &chooser->filetype, -1);
     gwy_app_file_chooser_update_expander(chooser);
-
-    /* XXX: Reset filter and set it again.  There is no way to notify the
-     * file chooser dialog the filter has changed. */
-    key = g_strconcat(chooser->prefix, "/filter", NULL);
-    gwy_container_gis_boolean_by_name(gwy_app_settings_get(), key, &active);
-    g_free(key);
-
-    if (active) {
-        gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(chooser),
-                                    chooser->no_filter);
-        gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(chooser),
-                                    chooser->filter);
-    }
 }
 
 static void
@@ -485,8 +479,8 @@ gwy_app_file_chooser_add_type_list(GwyAppFileChooser *chooser)
                            COLUMN_FILETYPE, "",
                            COLUMN_LABEL, _("Automatic by extension"),
                            -1);
-        gwy_app_file_chooser_add_types(store, GWY_FILE_OPERATION_SAVE);
-        gwy_app_file_chooser_add_types(store, GWY_FILE_OPERATION_EXPORT);
+        gwy_app_file_chooser_add_types(store, GWY_FILE_OPERATION_SAVE, FALSE);
+        gwy_app_file_chooser_add_types(store, GWY_FILE_OPERATION_EXPORT, FALSE);
         break;
 
         case GTK_FILE_CHOOSER_ACTION_OPEN:
@@ -494,7 +488,7 @@ gwy_app_file_chooser_add_type_list(GwyAppFileChooser *chooser)
                            COLUMN_FILETYPE, "",
                            COLUMN_LABEL, _("Automatically detected"),
                            -1);
-        gwy_app_file_chooser_add_types(store, GWY_FILE_OPERATION_LOAD);
+        gwy_app_file_chooser_add_types(store, GWY_FILE_OPERATION_LOAD, TRUE);
         break;
 
         default:
@@ -543,8 +537,7 @@ gwy_app_file_chooser_add_type_list(GwyAppFileChooser *chooser)
 
     if (action == GTK_FILE_CHOOSER_ACTION_OPEN) {
         chooser->filter_enable
-            = gtk_check_button_new_with_mnemonic(_("Show only loadable "
-                                                   "files of selected type"));
+            = gtk_check_button_new_with_mnemonic(_("Show only loadable files"));
         key = g_strconcat(chooser->prefix, "/filter", NULL);
         gwy_container_gis_boolean_by_name(gwy_app_settings_get(), key, &filter);
         g_free(key);
@@ -562,7 +555,7 @@ gwy_app_file_chooser_add_type_list(GwyAppFileChooser *chooser)
     /* Give it some reasonable size. FIXME: hack. */
     gtk_widget_show_all(vbox);
     gtk_widget_size_request(scwin, &req);
-    gtk_widget_set_size_request(scwin, -1, 3*req.height + 20);
+    gtk_widget_set_size_request(scwin, -1, 2*req.height + 20);
 
     gwy_app_file_chooser_select_type(chooser);
     gwy_app_file_chooser_type_changed(chooser, selection);
@@ -577,15 +570,8 @@ gwy_app_file_chooser_open_filter(const GtkFileFilterInfo *filter_info,
     gint score;
 
     chooser = GWY_APP_FILE_CHOOSER(userdata);
-    if (chooser->filetype && *chooser->filetype)
-        return gwy_file_func_run_detect(chooser->filetype,
-                                        filter_info->filename,
-                                        FALSE);
-
     name = gwy_file_detect_with_score(filter_info->filename,
-                                      FALSE,
-                                      GWY_FILE_OPERATION_LOAD,
-                                      &score);
+                                      FALSE, GWY_FILE_OPERATION_LOAD, &score);
     /* To filter out `fallback' importers like rawfile */
     return name != NULL && score >= 5;
 }
