@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyddion.h 8895 2008-12-28 12:19:29Z yeti-dn $
+ *  @(#) $Id: gwyddion.h 10267 2009-10-20 14:22:03Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -30,6 +30,13 @@ G_BEGIN_DECLS
 
 #define GWY_TOOLBOX_WM_ROLE "gwyddion-toolbox"
 
+#define REMOTE_NONE   0
+#define REMOTE_X11    1
+#define REMOTE_WIN32  2
+#define REMOTE_UNIQUE 3
+
+typedef struct _GwyRemote GwyRemote;
+
 typedef enum {
     GWY_APP_REMOTE_NONE = 0,
     GWY_APP_REMOTE_NEW,
@@ -37,9 +44,18 @@ typedef enum {
     GWY_APP_REMOTE_QUERY
 } GwyAppRemoteType;
 
-void       gwy_app_do_remote                (GwyAppRemoteType type,
-                                             int argc,
-                                             char **argv);
+void       gwy_remote_setup             (GtkWidget *toolbox);
+void       gwy_remote_finalize          (GtkWidget *toolbox);
+void       gwy_remote_do                (GwyAppRemoteType type,
+                                         int argc,
+                                         char **argv);
+GwyRemote* gwy_remote_get               (void);
+void       gwy_remote_free              (GwyRemote *remote);
+gboolean   gwy_remote_open_files        (GwyRemote *remote,
+                                         int argc,
+                                         char **argv);
+void       gwy_remote_print             (GwyRemote *remote);
+
 GtkWidget* gwy_app_toolbox_create           (void);
 GtkWidget* gwy_app_show_data_browser        (void);
 void       gwy_app_about                    (void);
