@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: menu.c 8196 2007-06-22 11:11:38Z yeti-dn $
+ *  @(#) $Id: menu.c 10529 2009-11-24 09:17:27Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -665,7 +665,9 @@ gwy_app_run_process_func_in_mode(const gchar *name,
         return;
 
     gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &data, 0);
-    g_return_if_fail(data);
+    g_return_if_fail(data
+                     || !(gwy_process_func_get_sensitivity_mask(name)
+                          & GWY_MENU_FLAG_DATA));
     gwy_process_func_run(name, data, run);
     gwy_app_update_last_process_func(name);
     gwy_app_sensitivity_set_state(GWY_MENU_FLAG_LAST_PROC,
@@ -850,6 +852,7 @@ gwy_app_menu_recent_files_update(GList *recent_files)
         }
         else {
             item = gtk_image_menu_item_new_with_mnemonic(label);
+            g_object_set(item, "always-show-image", TRUE, NULL);
             gwy_debug("creating item %p for <%s> [#%d]", item, s, i);
             g_object_set_qdata_full(G_OBJECT(item), quark, g_strdup(filename),
                                     g_free);
@@ -875,6 +878,7 @@ gwy_app_menu_recent_files_update(GList *recent_files)
         }
         else {
             item = gtk_image_menu_item_new_with_mnemonic("Thou Canst See This");
+            g_object_set(item, "always-show-image", TRUE, NULL);
             gwy_debug("adding hidden item %p [#%d]", item, i);
             gtk_menu_shell_append(GTK_MENU_SHELL(recent_files_menu), item);
             g_signal_connect(item, "activate",
