@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: settings.c 7499 2007-02-07 22:13:59Z yeti-dn $
+ *  @(#) $Id: settings.c 9274 2009-06-17 13:37:48Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -423,6 +423,7 @@ gwy_app_settings_apply(GwyContainer *settings)
     GwyResource *resource;
     const guchar *s;
     gchar **preferred, **p;
+    gboolean disabled;
 
     /* Preferred resources */
     if (gwy_container_gis_string_by_name(settings, "/app/gradients/preferred",
@@ -453,6 +454,12 @@ gwy_app_settings_apply(GwyContainer *settings)
     if (gwy_container_gis_string_by_name(settings, "/app/glmaterials/default",
                                          &s))
         gwy_inventory_set_default_item_name(gwy_gl_materials(), s);
+
+    /* Globally disabled 3D view axes */
+    if (gwy_container_gis_boolean_by_name(settings, "/app/3d/axes/disable",
+                                          &disabled)
+        && disabled)
+        gwy_3d_view_class_disable_axis_drawing(disabled);
 }
 
 /**
@@ -517,6 +524,11 @@ gwy_app_settings_get_settings_filename(void)
 gchar*
 gwy_app_settings_get_log_filename(void)
 {
+    const gchar *s;
+
+    if ((s = g_getenv("GWYDDION_LOGFILE")))
+        return g_strdup(s);
+
     return g_build_filename(gwy_get_user_dir(), "gwyddion.log", NULL);
 }
 
