@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyexpr.c 8479 2007-08-25 14:23:36Z yeti-dn $
+ *  @(#) $Id: gwyexpr.c 8874 2008-12-28 12:16:22Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -660,7 +660,7 @@ gwy_expr_token_new0(GwyExpr *expr)
     expr->reservoir = token->next;
     if (expr->reservoir)
         expr->reservoir->prev = NULL;
-    memset(token, 0, sizeof(GwyExprToken));
+    gwy_clear(token, 1);
 
     return token;
 }
@@ -1520,8 +1520,8 @@ gwy_expr_resolve_variables(GwyExpr *expr,
     g_return_val_if_fail(!n || (names && indices), 0);
 
     requested = g_newa(gboolean, expr->identifiers->len);
-    memset(requested, 0, expr->identifiers->len*sizeof(gboolean));
-    memset(indices, 0, n*sizeof(guint));
+    gwy_clear(requested, expr->identifiers->len);
+    gwy_clear(indices, n);
     for (i = 0; i < n; i++) {
         for (j = 1; j < expr->identifiers->len; j++) {
             if (gwy_strequal(names[i],
