@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwygrainvaluemenu.c 8490 2007-08-30 10:21:37Z yeti-dn $
+ *  @(#) $Id: gwygrainvaluemenu.c 10373 2009-11-07 19:59:04Z yeti-dn $
  *  Copyright (C) 2007 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -205,7 +205,7 @@ gwy_grain_value_tree_view_new(gboolean show_id,
         }
         else {
             g_warning("Unknown column `%s'", first_column);
-            title = "Unknonw";
+            title = "Unknown";
         }
 
         gtk_tree_view_column_set_title(column, title);
