@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwygraph.c 8472 2007-08-25 12:41:37Z yeti-dn $
+ *  @(#) $Id: gwygraph.c 8996 2009-02-06 10:15:24Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -348,7 +348,7 @@ gwy_graph_model_notify(GwyGraph *graph,
     }
 
     /* Units */
-    if (g_str_has_prefix(pspec->name, "si-unit")) {
+    if (g_str_has_prefix(pspec->name, "si-unit-")) {
         const gchar *name = pspec->name + strlen("si-unit-");
         GwySIUnit *unit = NULL;
 
