@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwysensitivitygroup.c 7384 2007-01-16 22:22:08Z yeti-dn $
+ *  @(#) $Id: gwysensitivitygroup.c 9102 2009-03-27 12:48:12Z yeti-dn $
  *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -209,7 +209,7 @@ gwy_sensitivity_group_add_widget(GwySensitivityGroup *sensgroup,
     /* Self-reference (pretend the widget has referenced us) */
     g_object_ref(sensgroup);
 
-    sens = ((senslist->mask & sensgroup->state) == senslist->mask);
+    sens = ((senslist->mask & sensgroup->old_state) == senslist->mask);
     gtk_widget_set_sensitive(widget, sens);
 }
 
