@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyprocessenums.c 9236 2009-06-11 07:51:09Z yeti-dn $
+ *  @(#) $Id: gwyprocessenums.c 10230 2009-10-12 13:14:27Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -243,7 +243,7 @@ gwy_windowing_type_get_enum(void)
         { N_("None"),       GWY_WINDOWING_NONE,      },
         { N_("Hann"),       GWY_WINDOWING_HANN,      },
         { N_("Hamming"),    GWY_WINDOWING_HAMMING,   },
-        { N_("Blackman"),   GWY_WINDOWING_BLACKMANN, },
+        { N_("Blackmann"),  GWY_WINDOWING_BLACKMANN, },
         { N_("Lanzcos"),    GWY_WINDOWING_LANCZOS,   },
         { N_("Welch"),      GWY_WINDOWING_WELCH,     },
         { N_("Rect"),       GWY_WINDOWING_RECT,      },
@@ -267,6 +267,41 @@ gwy_correlation_type_get_enum(void)
     return entries;
 }
 
+/**
+ * GwyMaskingType:
+ * @GWY_MASK_EXCLUDE: Exclude data under mask, i.e. take into account only
+ *                    data not covered by the mask.
+ * @GWY_MASK_INCLUDE: Take into account only data under the mask.
+ * @GWY_MASK_IGNORE: Ignore mask, if present, and use all data.
+ *
+ * Mask handling in procedures that can apply masking.
+ *
+ * Note at present many procedures do not have a masking argument and hence
+ * cannot apply masks in exclude mode.
+ *
+ * Since: 2.12
+ **/
+/**
+ * gwy_masking_type_get_enum:
+ *
+ * Returns #GwyEnum for #GwyMaskingType enum type.
+ *
+ * Returns: %NULL-terminated #GwyEnum which must not be modified nor freed.
+ *
+ * Since: 2.18
+ **/
+const GwyEnum*
+gwy_masking_type_get_enum(void)
+{
+    static const GwyEnum entries[] = {
+        { N_("Exclude masked region"),          GWY_MASK_EXCLUDE, },
+        { N_("Include only masked region"),     GWY_MASK_INCLUDE, },
+        { N_("Use entire image (ignore mask)"), GWY_MASK_IGNORE,  },
+        { NULL,                                 0,                },
+    };
+    return entries;
+}
+
 
 /************************** Documentation ****************************/
 
@@ -276,21 +311,6 @@ gwy_correlation_type_get_enum(void)
  * @short_description: Common enumerations
  **/
 
-/**
- * GwyMaskingType:
- * @GWY_MASK_EXCLUDE: Exclude data under mask, i.e. take into account only
- *                    data not covered by the mask.
- * @GWY_MASK_INCLUDE: Take into account only data under the mask.
- * @GWY_MASK_IGNORE: Ignore mask, if present, and use all data.
- *
- * Mask handling in procedures that can apply masking.
- *
- * FIXME: At present this is used namely by modules, the mask-application
- * interfaces of #GwyDataField methods are not very consistent.
- *
- * Since: 2.12
- **/
-
 /**
  * GwyTransformDirection:
  * @GWY_TRANSFORM_DIRECTION_BACKWARD: Backward (inverse) transform.
@@ -298,8 +318,8 @@ gwy_correlation_type_get_enum(void)
  *
  * Transform (namely integral transform) direction.
  *
- * In FFT, it is equal to sign of the exponent, that is backward transform
- * uses -1, forward transform +1.
+ * In FFT, it is equal to sign of the exponent, that is the backward transform
+ * uses -1, the forward transform +1.
  **/
 
 /**
@@ -448,7 +468,7 @@ gwy_correlation_type_get_enum(void)
  **/
 
 /**
- * GwyExteriorType;
+ * GwyExteriorType:
  * @GWY_EXTERIOR_UNDEFINED: The values corresponding to or calculated from
  *                          exterior data values are undefined, they may be
  *                          left unset or set to bogus values.  The caller
