@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: inttrans.c 8874 2008-12-28 12:16:22Z yeti-dn $
+ *  @(#) $Id: inttrans.c 9225 2009-06-08 20:06:36Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1748,10 +1748,10 @@ gwy_data_field_yfft_real_do(GwyDataField *rin,
         for (j = 1; j < rin->yres; j++) {
             gint n = rin->xres*rin->yres, kj = rin->xres*j;
 
-            r0[rin->xres*j] = (re[kj] + re[n - kj])/2.0;
-            i0[rin->xres*j] = (im[kj] - im[n - kj])/2.0;
-            r1[rin->xres*j] = (im[kj] + im[n - kj])/2.0;
-            i1[rin->xres*j] = (-re[kj] + re[n - kj])/2.0;
+            r0[kj] = (re[kj] + re[n - kj])/2.0;
+            i0[kj] = (im[kj] - im[n - kj])/2.0;
+            r1[kj] = (im[kj] + im[n - kj])/2.0;
+            i1[kj] = (-re[kj] + re[n - kj])/2.0;
         }
     }
     if (rin->xres % 2) {
