@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: simplefft.c 8874 2008-12-28 12:16:22Z yeti-dn $
+ *  @(#) $Id: simplefft.c 9224 2009-06-08 20:05:51Z yeti-dn $
  *  Copyright (C) 2003-2007 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1074,8 +1074,8 @@ gwy_fft_simple(GwyTransformDirection dir,
          * sign convention, unlike everything else here. */
         for (j = 0; j < k; j++)
             bluestein(m,
-                      k*istride, in_im + j, in_re + j,
-                      k*ostride, out_im + j, out_re + j);
+                      k*istride, in_im + j*istride, in_re + j*istride,
+                      k*ostride, out_im + j*ostride, out_re + j*ostride);
         if (k == 1)
             return;
     }
