@@ -1,5 +1,5 @@
 /*
- *  $Id: bcrfile.c 9037 2009-03-07 16:45:28Z yeti-dn $
+ *  $Id: bcrfile.c 10606 2009-12-02 10:44:00Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -42,10 +42,22 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Image Metrology BCR, BCRF
+ * .bcr .bcrf
+ * Read
+ **/
+
 #include "config.h"
 #include <string.h>
 #include <stdlib.h>
 #include <string.h>
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
 #include <libgwyddion/gwyutils.h>
@@ -82,10 +94,11 @@
 #define MAGIC5 "fileformat = bcrf\r\n"
 #define MAGIC_SIZE5 (sizeof(MAGIC5) - 1)
 
-
 #define MAGIC_SIZE \
     (MAX(MAX(MAX(MAGIC_SIZE1, MAGIC_SIZE2), MAX(MAGIC_SIZE3, MAGIC_SIZE4)), MAGIC_SIZE5))
 
+#define HEADER_SIZE_MAGIC "h\000e\000a\000d\000e\000r\000s\000i\000z\000e\000"
+
 /* values are bytes per pixel */
 typedef enum {
     BCR_FILE_INT16 = 2,
@@ -98,6 +111,8 @@ static gint          bcrfile_detect      (const GwyFileDetectInfo *fileinfo,
 static GwyContainer* bcrfile_load        (const gchar *filename,
                                           GwyRunType mode,
                                           GError **error);
+static void          find_header_size    (const guchar *buffer,
+                                          gsize *header_size);
 static GwyDataField* file_load_real      (const guchar *buffer,
                                           gsize size,
                                           gsize header_size,
@@ -110,8 +125,6 @@ static GwyDataField* read_data_field     (const guchar *buffer,
                                           BCRFileType type,
                                           gboolean little_endian,
                                           GwyDataField **voidmask);
-static void          load_metadata       (gchar *buffer,
-                                          GHashTable *meta);
 static GwyContainer* bcrfile_get_metadata(GHashTable *bcrmeta);
 
 static GwyModuleInfo module_info = {
@@ -119,7 +132,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Image Metrology BCR data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.11",
+    "0.13",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -172,6 +185,7 @@ bcrfile_load(const gchar *filename,
     gsize header_size, size = 0;
     GError *err = NULL;
     GwyDataField *dfield = NULL, *voidmask = NULL;
+    GwyTextHeaderParser parser;
     GHashTable *bcrmeta = NULL;
     gboolean utf16 = FALSE;
 
@@ -195,12 +209,13 @@ bcrfile_load(const gchar *filename,
         gunichar2 *s;
 
         header_size = 2*HEADER_SIZE;
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+        find_header_size(buffer, &header_size);
+#if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
         s = (gunichar2*)g_memdup(buffer, header_size);
 #endif
-#if G_BYTE_ORDER == G_BIG_ENDIAN
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
         s = g_new(gunichar2, HEADER_SIZE);
-        gwy_memcpy_byte_swap(buffer, s, sizeof(gunichar2), HEADER_SIZE, 1);
+        swab(buffer, s, header_size);
 #endif
         header = g_utf16_to_utf8(s, HEADER_SIZE, 0, 0, &err);
         g_free(s);
@@ -213,15 +228,18 @@ bcrfile_load(const gchar *filename,
 
             return NULL;
         }
+
     }
     else {
         header_size = HEADER_SIZE;
         header = g_memdup(buffer, header_size);
         header[header_size-1] = '\0';
     }
-    bcrmeta = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-    load_metadata(header, bcrmeta);
-    g_free(header);
+
+    gwy_clear(&parser, 1);
+    parser.comment_prefix = "#\n%";
+    parser.key_value_separator = "=";
+    bcrmeta = gwy_text_header_parse(header, &parser, NULL, NULL);
 
     dfield = file_load_real(buffer + header_size, size, header_size, bcrmeta,
                             &voidmask, error);
@@ -246,10 +264,48 @@ bcrfile_load(const gchar *filename,
         g_object_unref(meta);
     }
     g_hash_table_destroy(bcrmeta);
+    g_free(header);
 
     return container;
 }
 
+/* Newer files contain header size embedded in the header.  Change header_size
+ * if we find it. */
+static void
+find_header_size(const guchar *buffer,
+                 gsize *header_size)
+{
+    gsize size = 0;
+    const guchar *s;
+    guint16 c;
+
+    s = gwy_memmem(buffer, *header_size,
+                   HEADER_SIZE_MAGIC, sizeof(HEADER_SIZE_MAGIC)-1);
+    if (!s)
+        return;
+
+    s += sizeof(HEADER_SIZE_MAGIC)-1;
+    while (s - buffer < *header_size && (c = gwy_get_guint16_le(&s) == ' '))
+        ;
+    s -= sizeof(guint16);
+    while (s - buffer < *header_size && (c = gwy_get_guint16_le(&s) == '='))
+        ;
+    s -= sizeof(guint16);
+    while (s - buffer < *header_size && (c = gwy_get_guint16_le(&s) == ' '))
+        ;
+    s -= sizeof(guint16);
+    while (s - buffer < *header_size
+           && (c = gwy_get_guint16_le(&s))
+           && g_ascii_isdigit(c)) {
+        size *= 10;
+        size += c - '0';
+    }
+    if (size > 0) {
+        gwy_debug("header size found in the file: %lu", (gulong)size);
+        *header_size = 2*size;
+    }
+}
+
 static GwyDataField*
 file_load_real(const guchar *buffer,
                gsize size,
@@ -487,36 +543,4 @@ read_data_field(const guchar *buffer,
     return dfield;
 }
 
-static void
-load_metadata(gchar *buffer,
-              GHashTable *meta)
-{
-    gchar *line, *p;
-    gchar *key, *value;
-
-    while ((line = gwy_str_next_line(&buffer))) {
-        if (line[0] == '%' || line[0] == '#')
-            continue;
-
-        p = strchr(line, '=');
-        if (!p || p == line || !p[1])
-            continue;
-
-        key = g_strstrip(g_strndup(line, p-line));
-        if (!key[0]) {
-            g_free(key);
-            continue;
-        }
-        value = g_strstrip(g_strdup(p+1));
-        if (!value[0]) {
-            g_free(key);
-            g_free(value);
-            continue;
-        }
-
-        g_hash_table_insert(meta, key, value);
-        gwy_debug("<%s> = <%s>", key, value);
-    }
-}
-
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
