@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: burleigh.c 8916 2008-12-30 12:48:54Z yeti-dn $
+ *  @(#) $Id: burleigh.c 10195 2009-10-01 23:04:26Z yeti-dn $
  *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek, Hans-Peter Doerr.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net,
  *          doerr@cip.physik.uni-freiburg.de.
@@ -39,6 +39,13 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Burleigh IMG v2.1
+ * .img
+ * Read
+ **/
+
 #include "config.h"
 #include <string.h>
 #include <libgwyddion/gwymacros.h>
@@ -192,7 +199,7 @@ burleigh_load(const gchar *filename,
         return NULL;
     }
 
-    memset(&imgfile, 0, sizeof(imgfile));
+    gwy_clear(&imgfile, 1);
     p = buffer;
     imgfile.version = gwy_get_gfloat_le(&p);
     imgfile.version_int = GWY_ROUND(10*imgfile.version);
