@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: burleigh_exp.c 8916 2008-12-30 12:48:54Z yeti-dn $
+ *  @(#) $Id: burleigh_exp.c 10195 2009-10-01 23:04:26Z yeti-dn $
  *  Copyright (C) 2007 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -28,6 +28,13 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Burleigh exported data
+ * .txt .bin
+ * Read
+ **/
+
 #include "config.h"
 #include <stdlib.h>
 #include <string.h>
@@ -283,7 +290,7 @@ burleigh_exp_read_header(BurleighExpHeader *header,
     GwySIUnit *yunits = NULL, *dummy = NULL;
     gchar *p, *line;
 
-    memset(header, 0, sizeof(BurleighExpHeader));
+    gwy_clear(header, 1);
     p = buf;
 
     /* Magic header */
