@@ -1,5 +1,5 @@
 /*
- *  $Id: mapvue.c 9101 2009-03-25 10:25:57Z yeti-dn $
+ *  $Id: mapvue.c 10431 2009-11-14 07:36:16Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -30,10 +30,16 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * MapVue
+ * .map
+ * Read
+ **/
+
 #include "config.h"
 #include <string.h>
 #include <stdlib.h>
-#include <stdarg.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwyutils.h>
 #include <libgwyddion/gwymath.h>
