@@ -1,5 +1,5 @@
 /*
- *  $Id: nanoeducator.c 9186 2009-05-20 13:08:24Z yeti-dn $
+ *  $Id: nanoeducator.c 10431 2009-11-14 07:36:16Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -31,10 +31,16 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Nanoeducator
+ * .mspm .stm .spm
+ * Read
+ **/
+
 #include "config.h"
 #include <string.h>
 #include <stdlib.h>
-#include <stdarg.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwyutils.h>
 #include <libgwyddion/gwymath.h>
@@ -401,7 +407,7 @@ nanoedu_load(const gchar *filename,
         nobjects++;
     }
 
-    /* Additonal, but in fact alternative, data: one-line scans */
+    /* Additional, but in fact alternative, data: one-line scans */
     if (header.topo_nx && header.topo_ny
         && params.aqui_add == NANOEDU_SCANNER_TRAINING) {
         if (err_DIMENSION(error, header.topo_nx)
@@ -443,7 +449,7 @@ nanoedu_load(const gchar *filename,
         goto finish;
     }
 
-    /* Additonal data: spectra */
+    /* Additional data: spectra */
     if (params.aqui_spectr
         && params.n_spectra_lines && params.n_spectrum_points) {
         if (err_DIMENSION(error, params.n_spectra_lines)
@@ -472,7 +478,8 @@ nanoedu_load(const gchar *filename,
                                               Nanometer*q, scale*header.topo_ny,
                                               Nanometer*qx, Nanometer*qy,
                                               error);
-        else if (params.amp_modulation == 1)
+        else if ((params.amp_modulation == 1)
+              || (params.amp_modulation == 4))
             spectra = nanoedu_read_iz_spectra(buffer + header.point_offset,
                                               size - header.point_offset,
                                               buffer + header.spec_offset,
