@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: netcdf.c 8645 2007-10-17 13:34:16Z yeti-dn $
+ *  @(#) $Id: netcdf.c 10258 2009-10-17 20:20:51Z yeti-dn $
  *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -34,6 +34,13 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * GSXM NetCDF
+ * .nc
+ * Read
+ **/
+
 #include "config.h"
 #include <string.h>
 #include <libgwyddion/gwymacros.h>
@@ -167,7 +174,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports network Common Data Form (netCDF) files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.3",
+    "0.4",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -205,7 +212,7 @@ gxsm_detect(const GwyFileDetectInfo *fileinfo,
         return 0;
 
     score = 0;
-    memset(&cdffile, 0, sizeof(NetCDF));
+    gwy_clear(&cdffile, 1);
     p = fileinfo->head + MAGIC_SIZE;
     cdffile.nrecs = gwy_get_guint32_be(&p);
     if (cdffile_read_dim_array(&cdffile.dims, &cdffile.ndims,
@@ -253,8 +260,10 @@ gxsm_load(const gchar *filename,
         goto gxsm_load_fail;
     }
 
-    /* Look for variable "H".  This seems to be how GXSM calls data. */
-    if (!(var = cdffile_get_var(&cdffile, "H"))) {
+    /* Look for variable "H" or "FloatField".  This seems to be how GXSM calls
+     * data. */
+    if (!(var = cdffile_get_var(&cdffile, "H"))
+        && !(var = cdffile_get_var(&cdffile, "FloatField"))) {
         err_NO_DATA(error);
         goto gxsm_load_fail;
     }
@@ -308,7 +317,7 @@ gxsm_load(const gchar *filename,
         g_object_unref(siunit);
     }
     if ((siunit = read_real_size(&cdffile, "dz", &real, &power10))) {
-        /* on the other hand the units seem to be bogues here, take the range */
+        /* on the other hand the units seem to be bogus here, take the range */
         gwy_data_field_multiply(dfield, real);
         g_object_unref(siunit);
     }
@@ -458,7 +467,7 @@ cdffile_load(NetCDF *cdffile,
     GError *err = NULL;
     const guchar *p;
 
-    memset(cdffile, 0, sizeof(NetCDF));
+    gwy_clear(cdffile, 1);
 
     if (!gwy_file_get_contents(filename, &cdffile->buffer, &cdffile->size,
                                &err)) {
