@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: rawfile.c 9311 2009-06-25 08:14:24Z yeti-dn $
+ *  @(#) $Id: rawfile.c 10237 2009-10-13 09:33:28Z yeti-dn $
  *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -17,6 +17,21 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
  */
+
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Raw text files
+ * any
+ * Read
+ **/
+
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Raw binary files
+ * any
+ * Read
+ **/
+
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -214,7 +229,7 @@ static GwyModuleInfo module_info = {
     N_("Imports raw data files, both ASCII and binary, according to "
        "user-specified format."),
     "Yeti <yeti@gwyddion.net>",
-    "2.6",
+    "2.7",
     "David Nečas (Yeti) & Petr Klapetek",
     "2003",
 };
@@ -300,6 +315,9 @@ module_register(void)
                            (GwyFileLoadFunc)&rawfile_load,
                            NULL,
                            NULL);
+    /* We provide a detection function, but that's only to become the fallback
+     * loading method. */
+    gwy_file_func_set_is_detectable("rawfile", FALSE);
 
     return TRUE;
 }
@@ -584,15 +602,15 @@ rawfile_dialog_info_page(RawFileArgs *args,
 
     adj = gtk_adjustment_new(args->p.xres, 1, 16384, 1, 10, 0);
     controls->xres = gwy_table_attach_spinbutton(table, row,
-                                                 _("_Horizontal size:"),
-                                                 _("data samples"), adj);
+                                                 _("_Horizontal size:"), "px",
+                                                 adj);
     gtk_spin_button_set_digits(GTK_SPIN_BUTTON(controls->xres), 0);
     row++;
 
     adj = gtk_adjustment_new(args->p.yres, 1, 16384, 1, 10, 0);
     controls->yres = gwy_table_attach_spinbutton(table, row,
-                                                 _("_Vertical size:"),
-                                                 _("data samples"), adj);
+                                                 _("_Vertical size:"), "px",
+                                                 adj);
     gtk_spin_button_set_digits(GTK_SPIN_BUTTON(controls->yres), 0);
     row++;
 
@@ -2549,7 +2567,7 @@ rawfile_import_1x_presets(GwyContainer *settings)
         container = gwy_container_new();
         gwy_container_transfer(settings, container,
                                from->str, "/module/rawfile", TRUE);
-        memset(&args, 0, sizeof(RawFileArgs));
+        gwy_clear(&args, 1);
         rawfile_load_args(container, &args);
         preset = gwy_raw_file_preset_new(preset_list[i], &args.p, FALSE);
         gwy_inventory_insert_item(inventory, preset);
