@@ -1,5 +1,5 @@
 /*
- *  $Id: rawgraph.c 9089 2009-03-17 13:06:07Z yeti-dn $
+ *  $Id: rawgraph.c 10231 2009-10-12 13:20:48Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -18,6 +18,15 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
  */
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Graph text data (raw)
+ * any
+ * Read[1]
+ * [1] At present, only simple two-column data, imported as graph curves, are
+ * supported.
+ **/
+
 #include "config.h"
 #include <string.h>
 #include <gtk/gtk.h>
@@ -104,6 +113,9 @@ module_register(void)
                            (GwyFileLoadFunc)&rawgraph_load,
                            NULL,
                            NULL);
+    /* We provide a detection function, but the loading method tries a bit
+     * harder, so let the user choose explicitly. */
+    gwy_file_func_set_is_detectable("rawgraph", FALSE);
 
     return TRUE;
 }
@@ -516,11 +528,11 @@ static void
 rawgraph_load_args(GwyContainer *container,
                    RawGraphArgs *args)
 {
-    args->title = (gchar*)_("Curve");
-    args->x_label = (gchar*)"x";
-    args->y_label = (gchar*)"y";
-    args->x_units = (gchar*)"";
-    args->y_units = (gchar*)"";
+    args->title = NULL;
+    args->x_label = NULL;
+    args->y_label = NULL;
+    args->x_units = NULL;
+    args->y_units = NULL;
 
     gwy_container_gis_string_by_name(container, title_key,
                                      (const guchar**)&args->title);
@@ -533,11 +545,11 @@ rawgraph_load_args(GwyContainer *container,
     gwy_container_gis_string_by_name(container, y_units_key,
                                      (const guchar**)&args->y_units);
 
-    args->title = g_strdup(args->title);
-    args->x_label = g_strdup(args->x_label);
-    args->y_label = g_strdup(args->y_label);
-    args->x_units = g_strdup(args->x_units);
-    args->y_units = g_strdup(args->y_units);
+    args->title = g_strdup(args->title ? args->title : _("Curve"));
+    args->x_label = g_strdup(args->x_label ? args->x_label : "x");
+    args->y_label = g_strdup(args->y_label ? args->y_label : "y");
+    args->x_units = g_strdup(args->x_units ? args->x_units : "");
+    args->y_units = g_strdup(args->y_units ? args->y_units : "");
 }
 
 static void
