@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: sis.c 8950 2009-01-02 12:00:56Z yeti-dn $
+ *  @(#) $Id: sis.c 10195 2009-10-01 23:04:26Z yeti-dn $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -30,6 +30,13 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Surface Imaging Systems
+ * .sis
+ * Read
+ **/
+
 #include "config.h"
 #include <string.h>
 #include <libgwyddion/gwymacros.h>
@@ -505,7 +512,7 @@ sis_load(const gchar *filename,
         return NULL;
     }
     n = 0;
-    memset(&sisfile, 0, sizeof(sisfile));
+    gwy_clear(&sisfile, 1);
     sisfile.params = g_hash_table_new_full(g_direct_hash, g_direct_equal,
                                            NULL, g_free);
     if (sis_real_load(buffer, size, &sisfile, error)) {
