@@ -1,5 +1,5 @@
 /*
- *  $Id: spmlab.c 8916 2008-12-30 12:48:54Z yeti-dn $
+ *  $Id: spmlab.c 10739 2009-12-10 16:11:37Z yeti-dn $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -82,6 +82,13 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Thermicroscopes SPMLab R4-R7
+ * .tfr .ffr etc.
+ * Read
+ **/
+
 #include "config.h"
 #include <string.h>
 #include <libgwyddion/gwymacros.h>
@@ -112,7 +119,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Thermicroscopes SpmLab R3 to R7 data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.9",
+    "0.10",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -180,7 +187,7 @@ spmlab_load(const gchar *filename,
     gsize size = 0;
     GError *err = NULL;
     GwyDataField *dfield = NULL;
-    gchar *title;
+    gchar *title = NULL;
     gint dir;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
@@ -223,8 +230,7 @@ spmlab_load(const gchar *filename,
     g_object_unref(dfield);
 
     if (title)
-        gwy_container_set_string_by_name(container, "/0/data/title",
-                                         g_strdup(title));
+        gwy_container_set_string_by_name(container, "/0/data/title", title);
     else
         gwy_app_channel_title_fall_back(container, 0);
 
