@@ -1,5 +1,5 @@
 /*
- *  $Id: spmlabf.c 9037 2009-03-07 16:45:28Z yeti-dn $
+ *  $Id: spmlabf.c 10195 2009-10-01 23:04:26Z yeti-dn $
  *  Copyright (C) 2008 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,13 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Thermicroscopes SPMLab floating point
+ * .flt
+ * Read
+ **/
+
 #include "config.h"
 #include <string.h>
 #include <stdio.h>
@@ -57,7 +64,6 @@ static gint          slf_detect     (const GwyFileDetectInfo *fileinfo,
 static GwyContainer* slf_load       (const gchar *filename,
                                      GwyRunType mode,
                                      GError **error);
-static GHashTable*   read_hash      (gchar *buffer);
 static GwyContainer* add_metadata   (GHashTable *hash,
                                      ...);
 
@@ -66,7 +72,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports SPMLab floating-point files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.2",
+    "0.3",
     "David Nečas (Yeti)",
     "2008",
 };
@@ -106,6 +112,7 @@ slf_load(const gchar *filename,
          GError **error)
 {
     GwyContainer *meta = NULL, *container = NULL;
+    GwyTextHeaderParser parser;
     GHashTable *hash = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
@@ -114,7 +121,7 @@ slf_load(const gchar *filename,
     GwySIUnit *siunitx, *siunity, *siunitz, *siunit;
     const guchar *p;
     const gchar *val;
-    gchar *header, *end, *s;
+    gchar *header = NULL, *end, *s;
     gdouble *data, *row;
     guint data_offset, xres, yres, i, j;
     gdouble xreal, yreal, q, off;
@@ -143,8 +150,11 @@ slf_load(const gchar *filename,
 
     header = g_memdup(buffer, p - buffer + 1);
     header[p - buffer] = '\0';
-    hash = read_hash(header);
-    g_free(header);
+    /* Comment prefix [ means we ignore sections. */
+    gwy_clear(&parser, 1);
+    parser.comment_prefix = "[";
+    parser.key_value_separator = "=";
+    hash = gwy_text_header_parse(header, &parser, NULL, NULL);
 
     if (!require_keys(hash, error,
                       "DataOffset", "ScanRangeX", "ScanRangeY",
@@ -264,6 +274,7 @@ slf_load(const gchar *filename,
         gwy_container_set_object_by_name(container, "/0/meta", meta);
 
 fail:
+    g_free(header);
     if (hash)
         g_hash_table_destroy(hash);
     gwy_object_unref(meta);
@@ -273,37 +284,6 @@ fail:
     return container;
 }
 
-/* Read the ASCII header and fill the hash with key/value pairs */
-static GHashTable*
-read_hash(gchar *buffer)
-{
-    GHashTable *hash;
-    gchar *line, *eq, *p;
-
-    p = buffer;
-
-    hash = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-    while (p && p[0]) {
-        line = gwy_str_next_line(&p);
-        if (!line)
-            break;
-        eq = strchr(line, '=');
-        /* This also skips sections [Foo] */
-        if (!eq)
-            continue;
-        *eq = '\0';
-        if (*line != '\0') { /* drop entries without keyword */
-            eq++;
-            while (g_ascii_isspace(*eq))
-                eq++;
-            g_hash_table_insert(hash, g_strdup(line), g_strdup(eq));
-            gwy_debug("<%s>: <%s>", line, eq);
-        }
-    }
-
-    return hash;
-}
-
 static GwyContainer*
 add_metadata(GHashTable *hash,
              ...)
