@@ -1,5 +1,5 @@
 /*
- *  $Id: stmprg.c 8916 2008-12-30 12:48:54Z yeti-dn $
+ *  $Id: stmprg.c 10195 2009-10-01 23:04:26Z yeti-dn $
  *  Copyright (C) 2004 Rok Zitko
  *  E-mail: rok.zitko@ijs.si
  *
@@ -36,6 +36,13 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Omicron STMPRG
+ * tp* ta*
+ * Read
+ **/
+
 /* TODO
  * - store both directions
  * - other channels
@@ -607,7 +614,7 @@ stmprg_load(const gchar *filename,
         return NULL;
     }
 
-    memset(&stmprgfile, 0, sizeof(StmprgFile));
+    gwy_clear(&stmprgfile, 1);
     ok = read_parameters(buffer, size, &stmprgfile, error);
     gwy_file_abandon_contents(buffer, size, NULL);
     if (!ok)
