@@ -1,5 +1,5 @@
 /*
- *  $Id: unisoku.c 8950 2009-01-02 12:00:56Z yeti-dn $
+ *  $Id: unisoku.c 10195 2009-10-01 23:04:26Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -30,6 +30,13 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Unisoku
+ * .hdr .dat
+ * Read
+ **/
+
 #include "config.h"
 #include <string.h>
 #include <stdlib.h>
@@ -194,7 +201,7 @@ unisoku_load(const gchar *filename,
         return NULL;
     }
 
-    memset(&ufile, 0, sizeof(UnisokuFile));
+    gwy_clear(&ufile, 1);
     if (!unisoku_read_header(text, &ufile, error)) {
         unisoku_file_free(&ufile);
         g_free(text);
