@@ -1,5 +1,5 @@
 /*
- *  $Id: witfile.c 8950 2009-01-02 12:00:56Z yeti-dn $
+ *  $Id: witfile.c 10195 2009-10-01 23:04:26Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -27,6 +27,13 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * WITec
+ * .wit
+ * Read
+ **/
+
 #include "config.h"
 #include <string.h>
 #include <libgwyddion/gwymacros.h>
@@ -275,7 +282,7 @@ witec_load(const gchar *filename,
         return NULL;
     }
 
-    memset(&witfile, 0, sizeof(witfile));
+    gwy_clear(&witfile, 1);
     if (!witec_read_file(buffer, size, &witfile, error)) {
         gwy_file_abandon_contents(buffer, size, NULL);
         return NULL;
