@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: ellipse.c 7511 2007-02-11 18:49:21Z yeti-dn $
+ *  @(#) $Id: ellipse.c 9263 2009-06-16 09:56:04Z yeti-dn $
  *  Copyright (C) 2005-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -81,6 +81,14 @@ struct _GwySelectionEllipseClass {
 static gboolean module_register                  (void);
 static GType    gwy_layer_ellipse_get_type       (void) G_GNUC_CONST;
 static GType    gwy_selection_ellipse_get_type   (void) G_GNUC_CONST;
+static gboolean gwy_selection_ellipse_crop_object(GwySelection *selection,
+                                                  gint i,
+                                                  gpointer user_data);
+static void     gwy_selection_ellipse_crop       (GwySelection *selection,
+                                                  gdouble xmin,
+                                                  gdouble ymin,
+                                                  gdouble xmax,
+                                                  gdouble ymax);
 static void     gwy_layer_ellipse_set_property   (GObject *object,
                                                   guint prop_id,
                                                   const GValue *value,
@@ -130,7 +138,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Layer allowing selection of elliptic areas."),
     "Yeti <yeti@gwyddion.net>",
-    "1.5",
+    "1.6",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -155,6 +163,7 @@ gwy_selection_ellipse_class_init(GwySelectionEllipseClass *klass)
     GwySelectionClass *sel_class = GWY_SELECTION_CLASS(klass);
 
     sel_class->object_size = OBJECT_SIZE;
+    sel_class->crop = gwy_selection_ellipse_crop;
 }
 
 static void
@@ -205,6 +214,33 @@ gwy_selection_ellipse_init(GwySelectionEllipse *selection)
     g_array_set_size(GWY_SELECTION(selection)->objects, OBJECT_SIZE);
 }
 
+static gboolean
+gwy_selection_ellipse_crop_object(GwySelection *selection,
+                                  gint i,
+                                  gpointer user_data)
+{
+    const gdouble *minmax = (const gdouble*)user_data;
+    gdouble xy[OBJECT_SIZE];
+
+    gwy_selection_get_object(selection, i, xy);
+    return (MIN(xy[0], xy[2]) >= minmax[0]
+            && MIN(xy[1], xy[3]) >= minmax[1]
+            && MAX(xy[0], xy[2]) <= minmax[2]
+            && MAX(xy[1], xy[3]) <= minmax[3]);
+}
+
+static void
+gwy_selection_ellipse_crop(GwySelection *selection,
+                           gdouble xmin,
+                           gdouble ymin,
+                           gdouble xmax,
+                           gdouble ymax)
+{
+    gdouble minmax[4] = { xmin, ymin, xmax, ymax };
+
+    gwy_selection_filter(selection, gwy_selection_ellipse_crop_object, minmax);
+}
+
 static void
 gwy_layer_ellipse_init(G_GNUC_UNUSED GwyLayerEllipse *layer)
 {
