@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: point.c 8805 2008-11-28 10:21:37Z yeti-dn $
+ *  @(#) $Id: point.c 9284 2009-06-18 13:54:45Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -45,6 +45,7 @@ enum {
     PROP_0,
     PROP_DRAW_MARKER,
     PROP_MARKER_RADIUS,
+    PROP_DRAW_AS_VECTOR,
 };
 
 typedef struct _GwyLayerPoint          GwyLayerPoint;
@@ -61,6 +62,7 @@ struct _GwyLayerPoint {
     /* Properties */
     gboolean draw_marker;
     guint marker_radius;
+    gboolean draw_as_vector;
 };
 
 struct _GwyLayerPointClass {
@@ -75,39 +77,59 @@ struct _GwySelectionPointClass {
     GwySelectionClass parent_class;
 };
 
-static gboolean module_register                  (void);
-static GType    gwy_layer_point_get_type         (void) G_GNUC_CONST;
-static GType    gwy_selection_point_get_type     (void) G_GNUC_CONST;
-static void     gwy_layer_point_set_property     (GObject *object,
-                                                  guint prop_id,
-                                                  const GValue *value,
-                                                  GParamSpec *pspec);
-static void     gwy_layer_point_get_property     (GObject*object,
-                                                  guint prop_id,
-                                                  GValue *value,
-                                                  GParamSpec *pspec);
-static void     gwy_layer_point_draw             (GwyVectorLayer *layer,
-                                                  GdkDrawable *drawable,
-                                                  GwyRenderingTarget target);
-static void     gwy_layer_point_draw_object      (GwyVectorLayer *layer,
-                                                  GdkDrawable *drawable,
-                                                  GwyRenderingTarget target,
-                                                  gint i);
-static gboolean gwy_layer_point_motion_notify    (GwyVectorLayer *layer,
-                                                  GdkEventMotion *event);
-static gboolean gwy_layer_point_button_pressed   (GwyVectorLayer *layer,
-                                                  GdkEventButton *event);
-static gboolean gwy_layer_point_button_released  (GwyVectorLayer *layer,
-                                                  GdkEventButton *event);
-static void     gwy_layer_point_set_draw_marker  (GwyLayerPoint *layer,
-                                                  gboolean draw_marker);
-static void     gwy_layer_point_set_marker_radius(GwyLayerPoint *layer,
-                                                  guint radius);
-static void     gwy_layer_point_realize          (GwyDataViewLayer *dlayer);
-static void     gwy_layer_point_unrealize        (GwyDataViewLayer *dlayer);
-static gint     gwy_layer_point_near_point       (GwyVectorLayer *layer,
-                                                  gdouble xreal,
-                                                  gdouble yreal);
+static gboolean module_register                   (void);
+static GType    gwy_layer_point_get_type          (void) G_GNUC_CONST;
+static GType    gwy_selection_point_get_type      (void) G_GNUC_CONST;
+static gboolean gwy_selection_point_crop_object   (GwySelection *selection,
+                                                   gint i,
+                                                   gpointer user_data);
+static void     gwy_selection_point_crop          (GwySelection *selection,
+                                                   gdouble xmin,
+                                                   gdouble ymin,
+                                                   gdouble xmax,
+                                                   gdouble ymax);
+static void     gwy_layer_point_set_property      (GObject *object,
+                                                   guint prop_id,
+                                                   const GValue *value,
+                                                   GParamSpec *pspec);
+static void     gwy_layer_point_get_property      (GObject*object,
+                                                   guint prop_id,
+                                                   GValue *value,
+                                                   GParamSpec *pspec);
+static void     gwy_layer_point_draw              (GwyVectorLayer *layer,
+                                                   GdkDrawable *drawable,
+                                                   GwyRenderingTarget target);
+static void     gwy_layer_point_draw_object       (GwyVectorLayer *layer,
+                                                   GdkDrawable *drawable,
+                                                   GwyRenderingTarget target,
+                                                   gint i);
+static void     gwy_layer_point_draw_point        (GwyVectorLayer *layer,
+                                                   GdkDrawable *drawable,
+                                                   GwyDataView *data_view,
+                                                   GwyRenderingTarget target,
+                                                   const gdouble *xy);
+static void     gwy_layer_point_draw_vector       (GwyVectorLayer *layer,
+                                                   GdkDrawable *drawable,
+                                                   GwyDataView *data_view,
+                                                   GwyRenderingTarget target,
+                                                   const gdouble *xy);
+static gboolean gwy_layer_point_motion_notify     (GwyVectorLayer *layer,
+                                                   GdkEventMotion *event);
+static gboolean gwy_layer_point_button_pressed    (GwyVectorLayer *layer,
+                                                   GdkEventButton *event);
+static gboolean gwy_layer_point_button_released   (GwyVectorLayer *layer,
+                                                   GdkEventButton *event);
+static void     gwy_layer_point_set_draw_marker   (GwyLayerPoint *layer,
+                                                   gboolean draw_marker);
+static void     gwy_layer_point_set_marker_radius (GwyLayerPoint *layer,
+                                                   guint radius);
+static void     gwy_layer_point_set_draw_as_vector(GwyLayerPoint *layer,
+                                                   gboolean draw_as_vector);
+static void     gwy_layer_point_realize           (GwyDataViewLayer *dlayer);
+static void     gwy_layer_point_unrealize         (GwyDataViewLayer *dlayer);
+static gint     gwy_layer_point_near_point        (GwyVectorLayer *layer,
+                                                   gdouble xreal,
+                                                   gdouble yreal);
 
 /* Allow to express intent. */
 #define gwy_layer_point_undraw        gwy_layer_point_draw
@@ -119,7 +141,7 @@ static GwyModuleInfo module_info = {
     N_("Layer allowing selection of several points, displayed as crosses "
        "or inivisible."),
     "Yeti <yeti@gwyddion.net>",
-    "2.6",
+    "3.0",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -142,6 +164,7 @@ gwy_selection_point_class_init(GwySelectionPointClass *klass)
     GwySelectionClass *sel_class = GWY_SELECTION_CLASS(klass);
 
     sel_class->object_size = OBJECT_SIZE;
+    sel_class->crop = gwy_selection_point_crop;
 }
 
 static void
@@ -182,6 +205,15 @@ gwy_layer_point_class_init(GwyLayerPointClass *klass)
                            "Radius of marker to draw",
                            0, 50, 0,
                            G_PARAM_READWRITE));
+
+    g_object_class_install_property
+        (gobject_class,
+         PROP_DRAW_AS_VECTOR,
+         g_param_spec_boolean("draw-as-vector",
+                              "Draw as vector",
+                              "Whether to draw makers as lines from the origin",
+                              FALSE,
+                              G_PARAM_READWRITE));
 }
 
 static void
@@ -191,6 +223,31 @@ gwy_selection_point_init(GwySelectionPoint *selection)
     g_array_set_size(GWY_SELECTION(selection)->objects, OBJECT_SIZE);
 }
 
+static gboolean
+gwy_selection_point_crop_object(GwySelection *selection,
+                                gint i,
+                                gpointer user_data)
+{
+    const gdouble *minmax = (const gdouble*)user_data;
+    gdouble xy[OBJECT_SIZE];
+
+    gwy_selection_get_object(selection, i, xy);
+    return (xy[1] >= minmax[1] && xy[1] <= minmax[3]
+            && xy[0] >= minmax[0] && xy[0] <= minmax[2]);
+}
+
+static void
+gwy_selection_point_crop(GwySelection *selection,
+                         gdouble xmin,
+                         gdouble ymin,
+                         gdouble xmax,
+                         gdouble ymax)
+{
+    gdouble minmax[4] = { xmin, ymin, xmax, ymax };
+
+    gwy_selection_filter(selection, gwy_selection_point_crop_object, minmax);
+}
+
 static void
 gwy_layer_point_init(GwyLayerPoint *layer)
 {
@@ -215,6 +272,10 @@ gwy_layer_point_set_property(GObject *object,
         gwy_layer_point_set_marker_radius(layer, g_value_get_uint(value));
         break;
 
+        case PROP_DRAW_AS_VECTOR:
+        gwy_layer_point_set_draw_as_vector(layer, g_value_get_boolean(value));
+        break;
+
         default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -238,6 +299,10 @@ gwy_layer_point_get_property(GObject*object,
         g_value_set_uint(value, layer->marker_radius);
         break;
 
+        case PROP_DRAW_AS_VECTOR:
+        g_value_set_boolean(value, layer->draw_as_vector);
+        break;
+
         default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -271,9 +336,7 @@ gwy_layer_point_draw_object(GwyVectorLayer *layer,
                             gint i)
 {
     GwyDataView *data_view;
-    gint xc, yc, xmin, xmax, ymin, ymax, size, radius;
-    gint dwidth, dheight, xsize, ysize, xr, yr;
-    gdouble xy[OBJECT_SIZE], xreal, yreal, xm, ym;
+    gdouble xy[OBJECT_SIZE];
     gboolean has_object;
 
     g_return_if_fail(GDK_IS_DRAWABLE(drawable));
@@ -287,6 +350,23 @@ gwy_layer_point_draw_object(GwyVectorLayer *layer,
     has_object = gwy_selection_get_object(layer->selection, i, xy);
     g_return_if_fail(has_object);
 
+    if (GWY_LAYER_POINT(layer)->draw_as_vector)
+        gwy_layer_point_draw_vector(layer, drawable, data_view, target, xy);
+    else
+        gwy_layer_point_draw_point(layer, drawable, data_view, target, xy);
+}
+
+static void
+gwy_layer_point_draw_point(GwyVectorLayer *layer,
+                           GdkDrawable *drawable,
+                           GwyDataView *data_view,
+                           GwyRenderingTarget target,
+                           const gdouble *xy)
+{
+    gint xc, yc, xmin, xmax, ymin, ymax, size, radius;
+    gint dwidth, dheight, xsize, ysize, xr, yr;
+    gdouble xreal, yreal, xm, ym;
+
     radius = GWY_LAYER_POINT(layer)->marker_radius;
 
     gdk_drawable_get_size(drawable, &dwidth, &dheight);
@@ -333,6 +413,41 @@ gwy_layer_point_draw_object(GwyVectorLayer *layer,
                      xc - xr, yc - yr, 2*xr, 2*yr, 0, 64*360);
 }
 
+static void
+gwy_layer_point_draw_vector(GwyVectorLayer *layer,
+                            GdkDrawable *drawable,
+                            GwyDataView *data_view,
+                            GwyRenderingTarget target,
+                            const gdouble *xy)
+{
+    gint xi0, yi0, xi1, yi1, width, height;
+    gdouble xreal, yreal, xoff, yoff;
+
+    gwy_data_view_get_real_data_sizes(data_view, &xreal, &yreal);
+    gwy_data_view_get_real_data_offsets(data_view, &xoff, &yoff);
+    gdk_drawable_get_size(drawable, &width, &height);
+
+    switch (target) {
+        case GWY_RENDERING_TARGET_SCREEN:
+        gwy_data_view_coords_real_to_xy(data_view, -xoff, -yoff, &xi0, &yi0);
+        gwy_data_view_coords_real_to_xy(data_view, xy[0], xy[1], &xi1, &yi1);
+        gwy_data_view_coords_xy_cut_line(data_view, &xi0, &yi0, &xi1, &yi1);
+        break;
+
+        case GWY_RENDERING_TARGET_PIXMAP_IMAGE:
+        xi0 = floor(0.0*width/xreal);
+        yi0 = floor(0.0*height/yreal);
+        xi1 = floor(xy[0]*width/xreal);
+        yi1 = floor(xy[1]*height/yreal);
+        break;
+
+        default:
+        g_return_if_reached();
+        break;
+    }
+    gdk_draw_line(drawable, layer->gc, xi0, yi0, xi1, yi1);
+}
+
 static gboolean
 gwy_layer_point_motion_notify(GwyVectorLayer *layer,
                               GdkEventMotion *event)
@@ -380,9 +495,15 @@ gwy_layer_point_motion_notify(GwyVectorLayer *layer,
     }
 
     g_assert(layer->selecting != -1);
+    if (GWY_LAYER_POINT(layer)->draw_as_vector)
+        gwy_layer_point_undraw_object(layer, window,
+                                      GWY_RENDERING_TARGET_SCREEN, i);
     xy[0] = xreal;
     xy[1] = yreal;
     gwy_selection_set_object(layer->selection, i, xy);
+    if (GWY_LAYER_POINT(layer)->draw_as_vector)
+        gwy_layer_point_draw_object(layer, window,
+                                    GWY_RENDERING_TARGET_SCREEN, i);
 
     return FALSE;
 }
@@ -446,6 +567,10 @@ gwy_layer_point_button_pressed(GwyVectorLayer *layer,
         layer->selecting = gwy_selection_set_object(layer->selection, i, xy);
     }
     layer->button = event->button;
+    if (GWY_LAYER_POINT(layer)->draw_as_vector)
+        gwy_layer_point_draw_object(layer, window,
+                                    GWY_RENDERING_TARGET_SCREEN,
+                                    layer->selecting);
 
     gdk_window_set_cursor(window, GWY_LAYER_POINT(layer)->move_cursor);
     gwy_vector_layer_object_chosen(layer, layer->selecting);
@@ -482,6 +607,9 @@ gwy_layer_point_button_released(GwyVectorLayer *layer,
     gwy_data_view_coords_xy_clamp(data_view, &x, &y);
     outside = (event->x != x) || (event->y != y);
     gwy_data_view_coords_xy_to_real(data_view, x, y, &xreal, &yreal);
+    if (GWY_LAYER_POINT(layer)->draw_as_vector)
+        gwy_layer_point_undraw_object(layer, window,
+                                      GWY_RENDERING_TARGET_SCREEN, i);
     xy[0] = xreal;
     xy[1] = yreal;
     gwy_selection_set_object(layer->selection, i, xy);
@@ -544,6 +672,30 @@ gwy_layer_point_set_marker_radius(GwyLayerPoint *layer, guint radius)
     g_object_notify(G_OBJECT(layer), "marker-radius");
 }
 
+static void
+gwy_layer_point_set_draw_as_vector(GwyLayerPoint *layer,
+                                   gboolean draw_as_vector)
+{
+    GwyVectorLayer *vector_layer;
+    GtkWidget *parent;
+
+    g_return_if_fail(GWY_IS_LAYER_POINT(layer));
+    vector_layer = GWY_VECTOR_LAYER(layer);
+    parent = GWY_DATA_VIEW_LAYER(layer)->parent;
+
+    if (draw_as_vector == layer->draw_as_vector)
+        return;
+
+    if (parent && GTK_WIDGET_REALIZED(parent))
+        gwy_layer_point_undraw(vector_layer, parent->window,
+                               GWY_RENDERING_TARGET_SCREEN);
+    layer->draw_as_vector = draw_as_vector;
+    if (parent && GTK_WIDGET_REALIZED(parent))
+        gwy_layer_point_draw(vector_layer, parent->window,
+                             GWY_RENDERING_TARGET_SCREEN);
+    g_object_notify(G_OBJECT(layer), "draw-as-vector");
+}
+
 static void
 gwy_layer_point_realize(GwyDataViewLayer *dlayer)
 {
