@@ -1,5 +1,5 @@
 /*
- *  @(%) $Id: plugin-proxy.c 8690 2007-12-07 09:11:33Z yeti-dn $
+ *  @(%) $Id: plugin-proxy.c 10373 2009-11-07 19:59:04Z yeti-dn $
  *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -635,7 +635,7 @@ proc_find_plugin(const gchar *name,
         return NULL;
     }
     if (!(info->run & run)) {
-        g_critical("Plug-in `%s' doesn't suport this run mode.", name);
+        g_critical("Plug-in `%s' doesn't support this run mode.", name);
         return NULL;
     }
 
@@ -953,7 +953,7 @@ file_find_plugin(const gchar *name,
         return NULL;
     }
     if (!(info->run & run)) {
-        g_critical("Plug-in `%s' doesn't suport this operation.", name);
+        g_critical("Plug-in `%s' doesn't support this operation.", name);
         return NULL;
     }
 
