@@ -1,7 +1,7 @@
 /*
- *  @(#) $Id: arithmetic.c 8929 2008-12-31 13:40:16Z yeti-dn $
- *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
- *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
+ *  @(#) $Id: arithmetic.c 9610 2009-08-17 13:29:32Z yeti-dn $
+ *  Copyright (C) 2003-2009 David Necas (Yeti).
+ *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,9 @@
  */
 
 #include "config.h"
+#include <string.h>
+#include <stdio.h>
+#include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
@@ -32,7 +35,15 @@
 #define ARITH_RUN_MODES GWY_RUN_INTERACTIVE
 
 enum {
-    NARGS = 5
+    NARGS = 5,
+    HISTSIZE = 20
+};
+
+enum {
+    ARITHMETIC_VALUE,
+    ARITHMETIC_DER_X,
+    ARITHMETIC_DER_Y,
+    ARITHMETIC_NVARS,
 };
 
 enum {
@@ -41,6 +52,8 @@ enum {
     ARITHMETIC_EXPR = 2
 };
 
+typedef GwyDataField* (*MakeFieldFunc)(GwyDataField *dfield);
+
 typedef struct {
     GwyContainer *data;
     gint id;
@@ -49,10 +62,11 @@ typedef struct {
 typedef struct {
     GwyExpr *expr;
     gchar *expression;
+    GtkTreeModel *history;
     guint err;
     GwyDataObjectId objects[NARGS];
-    gchar *name[NARGS];
-    guint pos[NARGS];
+    gchar *name[NARGS*ARITHMETIC_NVARS];
+    guint pos[NARGS*ARITHMETIC_NVARS];
 } ArithmeticArgs;
 
 typedef struct {
@@ -63,22 +77,26 @@ typedef struct {
     GtkWidget *data[NARGS];
 } ArithmeticControls;
 
-static gboolean     module_register           (void);
-static void         arithmetic                (GwyContainer *data,
+static gboolean      module_register          (void);
+static void          arithmetic               (GwyContainer *data,
                                                GwyRunType run);
-static void         arithmetic_load_args      (GwyContainer *settings,
+static void          arithmetic_load_args     (GwyContainer *settings,
                                                ArithmeticArgs *args);
-static void         arithmetic_save_args      (GwyContainer *settings,
+static void          arithmetic_save_args     (GwyContainer *settings,
                                                ArithmeticArgs *args);
-static gboolean     arithmetic_dialog         (ArithmeticArgs *args);
-static void         arithmetic_data_cb        (GwyDataChooser *chooser,
+static gboolean      arithmetic_dialog        (ArithmeticArgs *args);
+static void          arithmetic_data_cb       (GwyDataChooser *chooser,
                                                ArithmeticControls *controls);
-static void         arithmetic_expr_cb        (GtkWidget *entry,
+static void          arithmetic_expr_cb       (GtkWidget *entry,
                                                ArithmeticControls *controls);
-static void         arithmetic_maybe_preview  (ArithmeticControls *controls);
-static const gchar* arithmetic_check          (ArithmeticArgs *args);
-static void         arithmetic_do             (ArithmeticArgs *args);
-
+static void          arithmetic_maybe_preview (ArithmeticControls *controls);
+static const gchar*  arithmetic_check         (ArithmeticArgs *args);
+static void          arithmetic_do            (ArithmeticArgs *args);
+static void          arithmetic_need_data     (const ArithmeticArgs *args,
+                                               gboolean *need_data);
+static void          arithmetic_update_history(ArithmeticArgs *args);
+static GwyDataField* make_x_der               (GwyDataField *dfield);
+static GwyDataField* make_y_der               (GwyDataField *dfield);
 
 static const gchar default_expression[] = "d1 - d2";
 
@@ -88,8 +106,8 @@ static GwyModuleInfo module_info = {
     N_("Simple arithmetic operations with two data fields "
        "(or a data field and a scalar)."),
     "Yeti <yeti@gwyddion.net>",
-    "2.3.1",
-    "David Nečas (Yeti) & Petr Klapetek",
+    "2.5",
+    "David Nečas (Yeti)",
     "2004",
 };
 
@@ -137,6 +155,8 @@ arithmetic(GwyContainer *data, GwyRunType run)
     }
     arithmetic_save_args(settings, &args);
     gwy_expr_free(args.expr);
+    for (i = 0; i < NARGS*ARITHMETIC_NVARS; i++)
+        g_free(args.name[i]);
 }
 
 static gboolean
@@ -170,6 +190,7 @@ arithmetic_dialog(ArithmeticArgs *args)
     row++;
 
     for (i = 0; i < NARGS; i++) {
+        /* VALUE is 0 */
         args->name[i] = g_strdup_printf("d_%d", i+1);
         label = gtk_label_new_with_mnemonic(args->name[i]);
         gwy_strkill(args->name[i], "_");
@@ -177,6 +198,9 @@ arithmetic_dialog(ArithmeticArgs *args)
         gtk_table_attach(GTK_TABLE(table), label, 0, 1, row, row+1,
                          GTK_EXPAND | GTK_FILL, 0, 0, 0);
 
+        args->name[NARGS*ARITHMETIC_DER_X + i] = g_strdup_printf("bx%d", i+1);
+        args->name[NARGS*ARITHMETIC_DER_Y + i] = g_strdup_printf("by%d", i+1);
+
         chooser = gwy_data_chooser_new_channels();
         gwy_data_chooser_set_active(GWY_DATA_CHOOSER(chooser),
                                     args->objects[i].data, args->objects[i].id);
@@ -198,8 +222,9 @@ arithmetic_dialog(ArithmeticArgs *args)
                      GTK_EXPAND | GTK_FILL, 0, 0, 0);
     row++;
 
-    controls.expression = entry = gtk_entry_new();
-    gtk_entry_set_text(GTK_ENTRY(entry), args->expression);
+    entry = gtk_combo_box_entry_new_with_model(args->history, 0);
+    controls.expression = entry;
+    gtk_combo_box_set_active(GTK_COMBO_BOX(controls.expression), 0);
     gtk_table_attach(GTK_TABLE(table), entry, 0, 2, row, row+1,
                      GTK_EXPAND | GTK_FILL, 0, 0, 0);
     g_signal_connect(entry, "changed",
@@ -259,13 +284,15 @@ arithmetic_expr_cb(GtkWidget *entry,
                    ArithmeticControls *controls)
 {
     ArithmeticArgs *args;
+    GtkComboBox *combo;
     guint nvars;
     gdouble v;
     gchar *s;
 
+    combo = GTK_COMBO_BOX(entry);
     args = controls->args;
     g_free(args->expression);
-    args->expression = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
+    args->expression = g_strdup(gtk_combo_box_get_active_text(combo));
     args->err = ARITHMETIC_OK;
 
     if (gwy_expr_compile(args->expr, args->expression, NULL)) {
@@ -280,7 +307,7 @@ arithmetic_expr_cb(GtkWidget *entry,
                                               GTK_RESPONSE_OK, FALSE);
         }
         else {
-            if (!gwy_expr_resolve_variables(args->expr, NARGS,
+            if (!gwy_expr_resolve_variables(args->expr, NARGS*ARITHMETIC_NVARS,
                                             (const gchar*const*)args->name,
                                             args->pos)) {
                 arithmetic_maybe_preview(controls);
@@ -330,12 +357,15 @@ arithmetic_check(ArithmeticArgs *args)
     GQuark quark;
     GwyDataField *dfirst, *dfield;
     GwyDataCompatibilityFlags diff;
+    gboolean need_data[NARGS];
 
     if (args->err & ARITHMETIC_EXPR)
         return NULL;
 
+    arithmetic_need_data(args, need_data);
+
     for (i = 0; i < NARGS; i++) {
-        if (args->pos[i]) {
+        if (need_data[i]) {
             first = i;
             break;
         }
@@ -351,7 +381,7 @@ arithmetic_check(ArithmeticArgs *args)
     quark = gwy_app_get_data_key_for_id(args->objects[first].id);
     dfirst = GWY_DATA_FIELD(gwy_container_get_object(data, quark));
     for (i = first+1; i < NARGS; i++) {
-        if (!args->pos[i])
+        if (!need_data[i])
             continue;
 
         data = args->objects[i].data;
@@ -383,11 +413,14 @@ arithmetic_check(ArithmeticArgs *args)
 static void
 arithmetic_do(ArithmeticArgs *args)
 {
+    static MakeFieldFunc derivers[ARITHMETIC_NVARS] = {
+        NULL, make_x_der, make_y_der,
+    };
     GwyContainer *data, *firstdata = NULL;
     GQuark quark;
-    GwyDataField *dfield, *result = NULL;
-    /* We know the expression can't contain more variables than NARGS */
-    const gdouble *d[NARGS + 1];
+    GwyDataField **data_fields, *dfield, *result = NULL;
+    const gdouble **d;
+    gboolean need_data[NARGS];
     gdouble *r = NULL;
     gboolean first = TRUE;
     guint n = 0, i;
@@ -395,15 +428,24 @@ arithmetic_do(ArithmeticArgs *args)
 
     g_return_if_fail(!args->err);
 
+    arithmetic_need_data(args, need_data);
+    /* We know the expression can't contain more variables */
+    data_fields = g_new0(GwyDataField*, NARGS*ARITHMETIC_NVARS);
+    d = g_new0(const gdouble*, NARGS*ARITHMETIC_NVARS + 1);
     d[0] = NULL;
+
+    /* First get all the fields we directly have */
     for (i = 0; i < NARGS; i++) {
-        if (!args->pos[i])
+        gwy_debug("dfield[%u]: %s", i, need_data[i] ? "NEEDED" : "not needed");
+        if (!need_data[i])
             continue;
 
         data = args->objects[i].data;
         quark = gwy_app_get_data_key_for_id(args->objects[i].id);
         dfield = GWY_DATA_FIELD(gwy_container_get_object(data, quark));
+        data_fields[i] = dfield;
         d[args->pos[i]] = gwy_data_field_get_data_const(dfield);
+        gwy_debug("d[%u] set to PRIMARY %u", args->pos[i], i);
         if (first) {
             first = FALSE;
             n = gwy_data_field_get_xres(dfield)*gwy_data_field_get_yres(dfield);
@@ -415,8 +457,31 @@ arithmetic_do(ArithmeticArgs *args)
     }
     g_return_if_fail(firstdata);
 
+    /* Derive derived fields */
+    for (i = NARGS; i < NARGS*ARITHMETIC_NVARS; i++) {
+        if (!args->pos[i])
+            continue;
+
+        g_return_if_fail(data_fields[i % NARGS]);
+        dfield = derivers[i/NARGS](data_fields[i % NARGS]);
+        gwy_debug("d[%u] set to DERIVED %u (type %u)",
+                  args->pos[i], i/NARGS, i%NARGS);
+        data_fields[i] = dfield;
+        d[args->pos[i]] = gwy_data_field_get_data_const(dfield);
+    }
+
+    /* Execute */
     gwy_expr_vector_execute(args->expr, n, d, r);
 
+    /* Free stuff */
+    for (i = NARGS; i < ARITHMETIC_NVARS; i++) {
+        if (data_fields[i])
+            g_object_unref(data_fields[i]);
+    }
+    g_free(data_fields);
+    g_free(d);
+
+    /* Create the new data field */
     newid = gwy_app_data_browser_add_data_field(result, firstdata, TRUE);
     g_object_unref(result);
     gwy_app_set_data_field_title(firstdata, newid, _("Calculated"));
@@ -424,26 +489,194 @@ arithmetic_do(ArithmeticArgs *args)
                             GWY_DATA_ITEM_GRADIENT, 0);
 }
 
+/* Find which data we need, for derivatives or otherwise */
+static void
+arithmetic_need_data(const ArithmeticArgs *args,
+                     gboolean *need_data)
+{
+    guint i;
+
+    gwy_clear(need_data, NARGS);
+    for (i = 0; i < NARGS*ARITHMETIC_NVARS; i++) {
+        if (args->pos[i])
+            need_data[i % NARGS] = TRUE;
+    }
+}
+
+static GwyDataField*
+make_x_der(GwyDataField *dfield)
+{
+    GwyDataField *result;
+    const gdouble *d, *drow;
+    gdouble *r, *rrow;
+    guint xres, yres, i, j;
+    gdouble h;
+
+    result = gwy_data_field_new_alike(dfield, FALSE);
+    xres = gwy_data_field_get_xres(dfield);
+    yres = gwy_data_field_get_yres(dfield);
+    h = 2.0*gwy_data_field_get_xreal(dfield)/xres;
+    d = gwy_data_field_get_data_const(dfield);
+    r = gwy_data_field_get_data(result);
+
+    if (xres < 2) {
+        gwy_data_field_clear(result);
+        return result;
+    }
+
+    for (i = 0; i < yres; i++) {
+        drow = d + i*xres;
+        rrow = r + i*xres;
+        rrow[0] = 2.0*(drow[1] - drow[0])/h;
+        for (j = 1; j < xres-1; j++)
+            rrow[j] = (drow[j+1] - drow[j-1])/h;
+        rrow[xres-1] = 2.0*(drow[xres-1] - drow[xres-2])/h;
+    }
+
+    return result;
+}
+
+static GwyDataField*
+make_y_der(GwyDataField *dfield)
+{
+    GwyDataField *result;
+    const gdouble *d, *drow, *drowp, *drowm;
+    gdouble *r, *rrow;
+    guint xres, yres, i, j;
+    gdouble h;
+
+    result = gwy_data_field_new_alike(dfield, FALSE);
+    xres = gwy_data_field_get_xres(dfield);
+    yres = gwy_data_field_get_yres(dfield);
+    h = 2.0*gwy_data_field_get_yreal(dfield)/yres;
+    d = gwy_data_field_get_data_const(dfield);
+    r = gwy_data_field_get_data(result);
+
+    if (yres < 2) {
+        gwy_data_field_clear(result);
+        return result;
+    }
+
+    rrow = r;
+    drow = d;
+    drowp = d + xres;
+    for (j = 0; j < xres; j++)
+        rrow[j] = 2.0*(drowp[j] - drow[j])/h;
+
+    for (i = 1; i < yres-1; i++) {
+        drowm = drow;
+        drow = drowp;
+        drowp += xres;
+        rrow += xres;
+        for (j = 0; j < xres; j++)
+            rrow[j] = (drowp[j] - drowm[j])/h;
+    }
+
+    drowm = drow;
+    drow = drowp;
+    rrow += xres;
+    for (j = 0; j < xres; j++)
+        rrow[j] = 2.0*(drow[j] - drowm[j])/h;
+
+    return result;
+}
+
+static void
+arithmetic_update_history(ArithmeticArgs *args)
+{
+    GtkListStore *store;
+    GtkTreeIter iter;
+    gchar *s;
+
+    if (!*args->expression)
+        return;
+
+    store = GTK_LIST_STORE(args->history);
+    gtk_list_store_prepend(store, &iter);
+    gtk_list_store_set(store, &iter, 0, args->expression, -1);
+
+    while (gtk_tree_model_iter_next(args->history, &iter)) {
+        gtk_tree_model_get(args->history, &iter, 0, &s, -1);
+        if (gwy_strequal(s, args->expression)) {
+            gtk_list_store_remove(store, &iter);
+            break;
+        }
+    }
+}
+
 static const gchar expression_key[] = "/module/arithmetic/expression";
 
 static void
 arithmetic_load_args(GwyContainer *settings,
                      ArithmeticArgs *args)
 {
+    GtkListStore *store;
     const guchar *exprstr;
+    gchar *filename, *buffer, *line, *p;
+    gsize size;
 
     exprstr = default_expression;
     gwy_container_gis_string_by_name(settings, expression_key, &exprstr);
     args->expression = g_strdup(exprstr);
+
+    store = gtk_list_store_new(1, G_TYPE_STRING);
+    args->history = GTK_TREE_MODEL(store);
+
+    filename = g_build_filename(gwy_get_user_dir(), "arithmetic", "history",
+                                NULL);
+    if (g_file_get_contents(filename, &buffer, &size, NULL)) {
+        p = buffer;
+        for (line = gwy_str_next_line(&p); line; line = gwy_str_next_line(&p)) {
+            GtkTreeIter iter;
+
+            g_strstrip(line);
+            if (*line) {
+                gtk_list_store_append(store, &iter);
+                gtk_list_store_set(store, &iter, 0, line, -1);
+            }
+        }
+        g_free(buffer);
+    }
+    g_free(filename);
+
+    /* Ensures args->expression comes first */
+    arithmetic_update_history(args);
 }
 
 static void
 arithmetic_save_args(GwyContainer *settings,
                      ArithmeticArgs *args)
 {
+    gchar *filename;
+    FILE *fh;
+
     gwy_container_set_string_by_name(settings, expression_key,
                                      args->expression);
+
+    filename = g_build_filename(gwy_get_user_dir(), "arithmetic", NULL);
+    if (!g_file_test(filename, G_FILE_TEST_IS_DIR))
+        g_mkdir(filename, 0700);
+    g_free(filename);
+
+    filename = g_build_filename(gwy_get_user_dir(), "arithmetic", "history",
+                                NULL);
+    if ((fh = g_fopen(filename, "w"))) {
+        GtkTreeIter iter;
+        guint i = 0;
+        gchar *s;
+
+        if (gtk_tree_model_get_iter_first(args->history, &iter)) {
+            do {
+                gtk_tree_model_get(args->history, &iter, 0, &s, -1);
+                fputs(s, fh);
+                fputc('\n', fh);
+                g_free(s);
+            } while (++i < HISTSIZE
+                     && gtk_tree_model_iter_next(args->history, &iter));
+        }
+        fclose(fh);
+    }
+    g_free(filename);
 }
 
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
-
