@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: convolution_filter.c 8390 2007-07-22 18:15:16Z yeti-dn $
+ *  @(#) $Id: convolution_filter.c 10297 2009-10-26 19:27:45Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -136,7 +136,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Generic convolution filter with a user-defined matrix."),
     "Yeti <yeti@gwyddion.net>",
-    "1.4",
+    "1.5",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -889,11 +889,13 @@ convolution_filter_fetch_coeff(ConvolutionControls *controls)
 }
 
 static void
-convolution_filter_hsym_changed(G_GNUC_UNUSED GtkToggleButton *button,
+convolution_filter_hsym_changed(GtkToggleButton *button,
                                 ConvolutionControls *controls)
 {
     if (controls->in_update)
         return;
+    if (!gtk_toggle_button_get_active(button))
+        return;
 
     controls->args->preset->hsym
         = gwy_radio_buttons_get_current(controls->hsym);
@@ -904,11 +906,13 @@ convolution_filter_hsym_changed(G_GNUC_UNUSED GtkToggleButton *button,
 }
 
 static void
-convolution_filter_vsym_changed(G_GNUC_UNUSED GtkToggleButton *button,
+convolution_filter_vsym_changed(GtkToggleButton *button,
                                 ConvolutionControls *controls)
 {
     if (controls->in_update)
         return;
+    if (!gtk_toggle_button_get_active(button))
+        return;
 
     controls->args->preset->vsym
         = gwy_radio_buttons_get_current(controls->vsym);
@@ -919,13 +923,15 @@ convolution_filter_vsym_changed(G_GNUC_UNUSED GtkToggleButton *button,
 }
 
 static void
-convolution_filter_size_changed(G_GNUC_UNUSED GtkToggleButton *button,
+convolution_filter_size_changed(GtkToggleButton *button,
                                 ConvolutionControls *controls)
 {
     guint newsize;
 
     if (controls->in_update)
         return;
+    if (!gtk_toggle_button_get_active(button))
+        return;
 
     newsize = gwy_radio_buttons_get_current(controls->sizes);
     gwy_convolution_filter_preset_data_resize(&controls->args->preset->data,
