@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: polylevel.c 8931 2008-12-31 13:41:31Z yeti-dn $
+ *  @(#) $Id: polylevel.c 10373 2009-11-07 19:59:04Z yeti-dn $
  *  Copyright (C) 2004,2008 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -132,7 +132,7 @@ module_register(void)
                               GWY_STOCK_POLYNOM,
                               POLYLEVEL_RUN_MODES,
                               GWY_MENU_FLAG_DATA,
-                              N_("Remove polynomial backgroud"));
+                              N_("Remove polynomial background"));
 
     return TRUE;
 }
@@ -498,15 +498,9 @@ poly_level_dialog(PolyLevelArgs *args,
         row++;
 
         controls.masking_group
-            = gwy_radio_buttons_createl(G_CALLBACK(poly_level_masking_changed),
-                                        &controls, args->masking,
-                                        _("_Exclude region under mask"),
-                                        GWY_MASK_EXCLUDE,
-                                        _("Exclude region _outside mask"),
-                                        GWY_MASK_INCLUDE,
-                                        _("Use entire _image (ignore mask)"),
-                                        GWY_MASK_IGNORE,
-                                        NULL);
+            = gwy_radio_buttons_create(gwy_masking_type_get_enum(), -1,
+                                       G_CALLBACK(poly_level_masking_changed),
+                                       &controls, args->masking);
         row = gwy_radio_buttons_attach_to_table(controls.masking_group,
                                                 GTK_TABLE(table), 3, row);
     }
