@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: filter.c 8993 2009-02-04 11:24:11Z yeti-dn $
+ *  @(#) $Id: filter.c 9084 2009-03-17 12:34:33Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -209,7 +209,7 @@ gwy_tool_filter_init_dialog(GwyToolFilter *tool)
         { N_("Maximum"),              GWY_FILTER_MAXIMUM,      },
         { N_("Kuwahara"),             GWY_FILTER_KUWAHARA,     },
         { N_("Dechecker"),            GWY_FILTER_DECHECKER,    },
-        { N_("Gaussian"),             GWY_FILTER_GAUSSIAN,     },
+        { N_("filter|Gaussian"),      GWY_FILTER_GAUSSIAN,     },
     };
     GtkDialog *dialog;
     GtkTable *table;
