@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: sfunctions.c 8340 2007-07-12 17:51:25Z yeti-dn $
+ *  @(#) $Id: sfunctions.c 8993 2009-02-04 11:24:11Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -21,6 +21,7 @@
 #include "config.h"
 #include <gtk/gtk.h>
 #include <libgwyddion/gwymacros.h>
+#include <libgwyddion/gwymath.h>
 #include <libgwymodule/gwymodule-tool.h>
 #include <libprocess/gwyprocesstypes.h>
 #include <libprocess/stats.h>
@@ -136,7 +137,7 @@ static GwyModuleInfo module_info = {
        "functions (height distribution, correlations, PSDF, Minkowski "
        "functionals) of selected part of data."),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "2.6",
+    "2.7",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -544,10 +545,10 @@ gwy_tool_sfunctions_update_curve(GwyToolSFunctions *tool)
             h = gwy_data_field_get_yres(plain_tool->data_field);
         }
         else {
-            isel[0] = gwy_data_field_rtoj(plain_tool->data_field, sel[0]);
-            isel[1] = gwy_data_field_rtoi(plain_tool->data_field, sel[1]);
-            isel[2] = gwy_data_field_rtoj(plain_tool->data_field, sel[2]);
-            isel[3] = gwy_data_field_rtoi(plain_tool->data_field, sel[3]);
+            isel[0] = floor(gwy_data_field_rtoj(plain_tool->data_field, sel[0]));
+            isel[1] = floor(gwy_data_field_rtoi(plain_tool->data_field, sel[1]));
+            isel[2] = floor(gwy_data_field_rtoj(plain_tool->data_field, sel[2]));
+            isel[3] = floor(gwy_data_field_rtoi(plain_tool->data_field, sel[3]));
 
             w = ABS(isel[2] - isel[0]) + 1;
             h = ABS(isel[3] - isel[1]) + 1;
